#! /usr/bin/perl

# ================================================================== #
#  Fumy Tiny URL Redirector   Ver 2.0.0   [r.cgi] (UTF-8)            #
# ================================================================== #
#  Copyright (C) Fumihiro Nishimura.(Nishishi) 2006-2016.            #
#                                                                    #
#  指定キーワードを使って指定されたURLへリダイレクトさせるCGIです。  #
#  転送キーワードとリダイレクト先URLは複数セットを登録できます。     #
#  長いURLを短く表現する目的などでも利用できるでしょう。             #
#                                                                    #
#  このCGIはどなたでも自由にご使用頂けます。個人・商用を問いません。 #
#  なお、著作権表示は削除・改変しないで下さい。                      #
#                                                                    #
#  http://www.nishishi.com/                                          #
# ================================================================== #

#use warnings;
use strict;
use CGI;

# ================ #
#  ▼ユーザ設定▼  #
# ================ #

# データファイル名（拡張子はcsv以外でも可）
my $datafile = 'redirects.csv';

# 初期画面に登録内容の一覧を表示するかどうか(1:する,0:しない)
my $listshow = 1;

# 登録鍵（＝登録用のパスワード）
my $password = 'sakura';

# 転送キーワードを自動生成する際の文字数(1以上の数値)
my $keywordchars = 5;

# 文字コード（このCGIとデータファイルの両方で共通の文字コードを使って下さい／デフォルトは「UTF-8」）
my $charcode = 'UTF-8';

# ---------------------------------- #
# ▼これ以降に設定項目はありません▼ #
# ---------------------------------- #
my $demo_mode = 0;	# デモ用フラグ(1:ファイル出力なし/0:通常使用)
my $BACKTOLINK = q|<a href="/" onclick="history.back();">直前のページに戻る</a>|;
my @REQCHARS = (0 .. 9 , 'A' .. 'Z', 'a' .. 'z');	# 転送キーワードに使える候補文字列(英数字)

# 値の不正チェック・修正
if( $listshow > 1 ) { $listshow = 1; }
elsif( $listshow < 0 ) { $listshow = 0; }
if( $keywordchars < 1 ) { $keywordchars = 5; }

# メイン処理
my @rdata;		# 短縮データ格納用
&loaddata();	# データファイルを読む
my $param = &getparam();	# パラメータを調べる
print 'Location: ' . &forsafety(&checkurl(&searchmatchdata("$param"))) . "\n\n";
exit;

# ---------------- #
# パラメータの取得 #
# ---------------- #
sub getparam
{
	my $query = $ENV{'QUERY_STRING'} || '';
	my $poslen = $ENV{'CONTENT_LENGTH'} || 0;

	# クエリ内容の確認
	if( $query eq '' && $poslen == 0 ) {
		# クエリがなければ初期画面の表示
		&startdisp();
	}
	elsif( $query =~ /=/ || $poslen > 0 ) {
		# クエリに等号が含まれているか、POSTで何か情報があれば、各種設定処理
		my $cgi = new CGI;
		my $showmode = $cgi->param('mode') || '';		# 動作選択: edit / try
		my $edititemnum = $cgi->param('edit') || 0;		# 編集対象番号: 0以上の数値

		my $tryeditnum = $cgi->param('editnum') || 0;		# 登録申請: 登録対象番号(0以上の数値)
		my $trykeyword = $cgi->param('keyword') || '';		# 登録申請: キーワード(半角英数/空文字)
		my $tryrediurl = $cgi->param('redirecturl') || '';	# 登録申請: URL
		my $tryregikey = $cgi->param('registkey') || '';	# 登録申請: 登録鍵

		# 値の不正チェック
		if( $edititemnum !~ m/\d+/ || $edititemnum < 0 ) { &errorexit("パラメータが不正です。edit番号は0以上の数値でのみ指定できます。"); }
		if( $tryeditnum !~ m/\d+/ || $edititemnum < 0 ) { &errorexit("値が不正です。登録対象番号は0以上の数値でのみ指定できます。"); }

		# 編集モード
		if( $showmode eq 'edit' ) {
			&editpage( $edititemnum );
			exit;
		}
		elsif( $showmode eq 'try' ) {
			if( $query eq '' ) {
				# 登録申請処理(POSTのみ)
				if( &checkregkey( $tryregikey ) ) {
					# 登録鍵が有効なら登録申請処理
					&tryregistration( $tryeditnum, $trykeyword, $tryrediurl );
				}
			}
			else {
				# GETだと拒否
				&errorexit("この処理はPOSTで送信されないと実行できません。");
			}
			exit;
		}
		else {
			&errorexit("パラメータが不正です。");
		}
	}

	# クエリに何かあって「=」が含まれていなければ転送キーワードとして解釈。
	return $query;
}

# ---------------- #
# データの読み込み #	グローバル変数 @rdata へ格納
# ---------------- #
sub loaddata
{
	@rdata = ();	# 現在の内容をクリア

	# データファイルの中身を1行ずつチェックしつつ、問題のないデータだけを読み込む
	if( open(IN,"$datafile") ) {
		foreach my $oneline ( <IN> ) {
			if( $oneline =~ m/^[a-zA-Z0-9]+,(.+)/ ) {
				if( $1 =~ m/\w/ ) {
					# データ条件に合致していれば追加（転送キーワードが英数字のみで、転送先URLに英数字が1文字以上あれば）
					push( @rdata, $oneline );
				}
			}
		}
		close(IN);
	}
	else {
		# 読めなかったらエラー
		&errorexit("データファイルが読めませんでした。存在を確認して下さい。");
	}
}

# ---------------- #
# データの書き込み #	グローバル変数 @rdata を使用／返り値: [0]書き込み失敗、[1以上]書き込み完了
# ---------------- #
sub savedata
{
	my ($editnum,$keyword,$redirecturl) = @_;

	# デモモードなら出力しない
	if( $demo_mode == 1 ) {
		return 0;
	}

	# editnum が0なら、新規追加。／editnum があれば、その番号だけを更新。
	if( $editnum == 0 ) {
		# 新規追加
		push( @rdata, "$keyword,$redirecturl\n" );
	}
	else {
		# 更新or削除
		if( $keyword ne '' ) {
			# 更新
			$rdata[$editnum-1] = "$keyword,$redirecturl\n";
		}
		else {
			# 削除
			$rdata[$editnum-1] = "";
		}
	}

	# 辞書順でソート
	@rdata = sort {$a cmp $b} @rdata;

	# 出力
	if( open( DATOUT, ">$datafile" ) ) {
		# ファイルが書き込みできる状態なら出力
		print DATOUT @rdata;
		close DATOUT;
	}
	else {
		return 0;
	}

	return 1;
}

# -------------------- #
# ランダム文字列の生成 #	引数：文字数
# -------------------- #	候補文字列はグローバル配列 @REQCHARS を使う。
sub getrandomchars
{
	my $requirenum = shift @_;
	if( $requirenum < 1 ) { $requirenum = 1; }	# 必要文字数(最低1文字)

	my $ret = '';		# 生成文字列格納用

	my $counter = 0;
	while( $counter < $requirenum ) {
		# 必要文字数に達するまでループ
		my $rnum = int(rand($#REQCHARS));	# 候補文字列からランダムに1つ選ぶ
		$ret .= $REQCHARS[$rnum];
		$counter++;
	}

	return $ret;
}

# ------------------------ #
# 転送キーワードの自動生成 #	生成文字数はグローバル変数 $keywordchars を使う。
# ------------------------ #
sub createkeyword
{
	my $newkeyword;

	# 生成文字数は1～10の範囲
	if( $keywordchars < 1 ) { $keywordchars = 1; }
	elsif( $keywordchars > 10 ) { $keywordchars = 10; }

	# 最大試行回数
	my $maxloop = $keywordchars * $#REQCHARS;

	my $counter = 0;
	while ( $counter < $maxloop ) {
		# 生成
		$newkeyword = &getrandomchars( $keywordchars );
		# 生成物の既存チェック
		if( &checkduplication($newkeyword,0,0) == 0 ) {
			# 既存キーワードと一致しなければ採用（ループ離脱）
			last;
		}
		$counter++;
	}

	# オーバーフロー
	if( $counter == $maxloop ) {
		&errorexit("転送キーワードを自動生成できませんでした。使用可能な組み合わせを使い切った可能性があります。(その場合は、文字数の設定値を増やしてみて下さい。)");
		exit;
	}

	return $newkeyword;
}

# ---------------- #
# データの一致確認 #
# ---------------- #
sub searchmatchdata
{
	my $checkstring = shift @_;

	# データファイルを呼んで一致項目を探す
	my $redurl = "";

	foreach my $oneline ( @rdata ) {
		if( $oneline =~ m/^$checkstring,(.*)/ ) {
			# 見つかったらループを抜ける
			$redurl = $1;
			last;
		}
	}

	return $redurl;
}

# ---------------- #
# 登録鍵の一致確認 #	正しければ 1 を返す。誤っていればエラーを表示して終了。
# ---------------- #
sub checkregkey
{
	my $trykey = shift @_;

	if( $password eq $trykey ) {
		return 1;
	}
	else {
		&showadminpage("ERROR",'登録鍵の不一致','<p class="errorguide">登録鍵が正しくありません。処理は実行できませんでした。</p><p>※登録鍵を確認して再入力して下さい。「登録鍵」はCGIソースの先頭付近で設定できるパスワード文字列です。</p>',$BACKTOLINK);
		exit;
	}
}

# ---------------------- #　※ここでは引数で与えられた情報を表示するだけ。データは読まない。
# 転送登録フォームの表示 #	新規作成時：パラメータなし
# ---------------------- #　編集時：パラメータ(No,Keyword,URL)
sub showregistform
{
	my ($editnum,$keyword,$redirecturl) = @_;
	my $msg = "";
	my $kind;
	my $add = "";

	if( $editnum > 0 ) {
		# 編集番号が指定されていれば編集
		$kind = "登録No.$editnumの編集";
		$add = qq|<a href="#" onclick="getElementById('keyword').value = ''; getElementById('keyword').style.backgroundColor = '#ddd'; getElementById('redirecturl').value = ''; getElementById('redirecturl').style.backgroundColor = '#ddd'; getElementById('submitbutton').value = '削除する'; getElementById('deleteguide').style.display = 'inline'; this.style.display = 'none'; getElementById('registkey').focus(); return false;" class="submenuitem" id="fordelete">このデータを削除する</a><span class="submenuitem" style="display:none;" id="deleteguide">※登録鍵だけを入力して「削除する」ボタンを押して下さい。</span>|;
	}
	else {
		$kind = "新規";
		$editnum = 0;
		$add = qq|<!-- a href="?mode=multipleadd" class="submenuitem">複数個を一括登録したい場合はこちら</a -->|;
	}

	# デモ
	my $demomsg = '';
	if( $demo_mode == 1 ) {
		$demomsg = qq|<p style="color: #cc0000; text-align:right; margin-top:0px;"><span class="demo">DEMO MODE</span> 動作サンプルです。</p>|;
	}

	# 表示前の安全対策
	$editnum = &forsafety($editnum);
	$keyword = &forsafety($keyword);
	$redirecturl = &forsafety($redirecturl);

	# 登録フォームの表示
	$msg .= qq|$demomsg
		<form action="?" method="post">
			<fieldset id="registform" class="fieldform">
				<legend>転送登録／$kind</legend>
				<p><label for="keyword"    >転送キーワード：</label>	<input type="text"     name="keyword"     id="keyword"     value="$keyword">		<span class="inputguide">（※半角英数字／省略するとランダムな$keywordchars文字を付与）</span></p>
				<p><label for="redirecturl">転送先URL：</label>			<input type="text"     name="redirecturl" id="redirecturl" value="$redirecturl">	<span class="inputguide"></span></p>
				<p><label for="registkey"  >登録鍵：</label>			<input type="password" name="registkey"   id="registkey"   value="">				<span class="inputguide">（※CGIソースで設定したパスワード）</span></p>
				<p><input type="hidden" name="editnum" value="$editnum"><input type="hidden" name="mode" value="try"><input type="submit" value="転送設定を登録する" id="submitbutton"> $add</p>
			</fieldset>
		</form>
	|;

	# 編集番号が指定されていれば使用するためのリンクを追加表示
	if( $editnum > 0 ) {
		$msg .= &showuselink( $keyword, $redirecturl );
	}

	return $msg;
}

# -------------------------- #
# 転送データ使用のリンク表示 #	引数：安全対策済みのkeyword
# -------------------------- #
sub showuselink {
	my $safekeyword = shift @_;
	my $ret = '';

	$ret .= qq|
		<fieldset id="uselink" class="fieldform">
			<legend>この転送設定を使うには</legend>
			<p>この転送設定を使いたい場合は、下記のリンク先をコピーしてお使い下さい。<small>(右クリックして「リンク先URLをコピー」などのメニューをお使い下さい。)</small></p>
			<p id="showuselink">→ <a href="?$safekeyword">$safekeyword</a></p>
			<p>上記のリンクをそのままクリックすると、設定された転送処理が実行されます。</p>
		</fieldset>
	|;

	return $ret;
}

# ---------------------- #
# 登録申請内容のチェック #	引数：editnum / keyword / redirecturl
# ---------------------- #
sub tryregistration
{
	my ($editnum,$keyword,$redirecturl) = @_;
	my $msg = '';
	my $errorcount = 0;

	# 実行フラグ
	my $dokeywordcheck = 0;	# キーワードの既存重複確認(0:パス,1:実行)
	my $dourlcheck = 0;		# URLの既存重複確認(0:パス,1:実行)
	my $doupdatedata = 0;	# データファイルの更新(0:パス,1:実行)

	# --- --- --- --- --- --- --- --- ---
	# ▼editnumを調べて新規か修正かを判断
	# --- --- --- --- --- --- --- --- ---
	if( $editnum == 0 ) {
		# 新規登録
		if( $keyword eq '' && $redirecturl eq '' ) {
			# 入力データがなければ、登録データ一覧だけの指示
			$dokeywordcheck = 0;
			$dourlcheck = 0;
			$doupdatedata = 0;
		}
		elsif( $keyword eq '' && $redirecturl ne '' ) {
			# キーワードはないが、URLはあるなら、キーワードを自動生成して登録
			$dokeywordcheck = 2;
			$dourlcheck = 1;
			$doupdatedata = 1;
		}
		elsif( $keyword ne '' && $redirecturl ne '' ) {
			# キーワードもURLもあるなら、登録
			$dokeywordcheck = 1;
			$dourlcheck = 1;
			$doupdatedata = 1;
		}
		else {
			# その他（＝キーワードはあるがURLがない）はエラー
			$dokeywordcheck = 0;
			$dourlcheck = 0;
			$doupdatedata = 0;
			$msg .= qq|<p class="errorguide">転送先URLが入力されていません。</p>|;
			$errorcount++;
		}
	}
	elsif(( $editnum > 0 ) && ( $editnum <= ($#rdata + 1) )) {
		# 既存情報の変更
		if( $keyword eq '' && $redirecturl eq '' ) {
			# 入力データがなければ、削除の指示
			$dokeywordcheck = 0;
			$dourlcheck = 0;
			$doupdatedata = 1;
		}
		elsif( $keyword eq '' && $redirecturl ne '' ) {
			# キーワードはないが、URLはあるなら、キーワードを自動生成して更新
			$dokeywordcheck = 2;
			$dourlcheck = 1;
			$doupdatedata = 1;
		}
		elsif( $keyword ne '' && $redirecturl ne '' ) {
			# キーワードもURLもあるなら、更新
			$dokeywordcheck = 1;
			$dourlcheck = 1;
			$doupdatedata = 1;
		}
		else {
			# その他（＝キーワードはあるがURLがない）はエラー
			$dokeywordcheck = 0;
			$dourlcheck = 0;
			$doupdatedata = 0;
			$msg .= qq|<p class="errorguide">転送先URLが入力されていません。このデータを削除したい場合は、転送キーワードも消して送信して下さい。</p>|;
			$errorcount++;
		}
	}
	else {
		# 範囲外
		&errorexit("※tryregistrationに渡された引数editnumの値がおかしい。");
	}

	# --- --- ---
	#keywordを調べて使用可能かチェック
	if( $dokeywordcheck == 1 ) {
		my $resk = &checkduplication($keyword,0,$editnum);
		if( $resk > 0 ) {
			# 既存のキーワードが指定されていた場合
			$msg .= qq|<p class="errorguide">転送キーワード「| . &forsafety($keyword) . qq|」は、既に <a href="?mode=edit&amp;edit=$resk">No. $resk</a> で使用済みです。他のキーワードを指定して下さい。</p>|;
			$errorcount++;
		}
		if( $keyword =~ m/[^a-zA-Z0-9]/ ) {
			# 使用可能文字以外が指定されていた場合
			$msg .= qq|<p class="errorguide">転送キーワードには、英数字のみが使用可能です。他のキーワードを指定して下さい。</p>|;
			$errorcount++;
		}
	}
	# キーワードの自動生成
	elsif( $dokeywordcheck == 2 ) {
		# キーワード自動生成
		$keyword = &createkeyword();
	}

	# --- --- ---
	#redirecturlが既存データに含まれていないかチェック
	if( $dourlcheck == 1 ) {
		my $resu = &checkduplication($redirecturl,1,$editnum);
		if( $resu > 0 ) {
			# 既存のURLが指定されていた場合
			$msg .= qq|<p class="errorguide">転送先URL「<span class="url">| . &forsafety($redirecturl) . qq|</span>」は、既に <a href="?mode=edit&amp;edit=$resu">No. $resu</a> に登録済みです。新たに登録する必要がありません。</p>|;
			$errorcount++;
		}
	}

	# 問題の有無
	if( $errorcount == 0 ) {
		# 問題がなければ
		if( $doupdatedata == 1 ) {
			# 登録更新処理
			if( &savedata($editnum,$keyword,$redirecturl) != 0 ) {
				# 書き込みができれば：データを再読込
				&loaddata();
				# 報告を表示
				$msg .= qq|<p class="successreport">データファイルを更新しました。</p>|;
				if( $keyword ne '' && $redirecturl ne '' ) {
					# キーワードと転送先があれば登録内容を表示
					$keyword = &forsafety($keyword);
					$redirecturl = &forsafety($redirecturl);
					$msg .= qq|<p>下記の転送設定を登録しました。</p><ul class="newrecord"><li>転送キーワード：<strong>$keyword</strong></li><li>転送先URL：<strong>$redirecturl</strong></li></ul>|;
					# 使用リンクも出力
					$msg .= &showuselink( $keyword, $redirecturl );
				}
				else {
					# キーワードも転送先がなければ削除処理なので結果を表示
					$msg .= qq|<p class="newrecord">指定の転送設定を削除しました。</p>|;
				}
				$msg .= qq|<p>※データファイルの中身は毎回辞書順にソートされるため、登録番号は毎回変化する可能性があります。</p>|;
			}
			else {
				# 書き込みできなければ：エラー報告を表示（強制終了はしない）
				$msg .= qq|<p class="errorguide">データファイルを更新できませんでした。</p><p>データファイルへの書き込みが禁止されている可能性があります。</p>|;
				if( $demo_mode == 1 ) {
					# デモモードなら
					$msg .= qq|<p class="demo"><span class="demo">DEMO MODE</span> このCGIは動作サンプルとして「デモモード」で動作しているため、ファイルへの出力は実行されませんでした。</p>|;
				}
			}
		}
		else {
			# 更新処理をしない場合
			$msg .= qq|<p class="successreport">現在の登録状況を表示します。</p>|;
		}

		# 登録内容を表示
		my @listhtml = &showlist();
		$msg .= join("", @listhtml) . "\n" . '';
		&showadminpage("Registered Successfully",'',$msg,'<a href="?">初期画面に戻る</a>');
	}
	else {
		#問題があれば、入力フォームを再表示
		$msg .= "<p>上記に示すとおり、$errorcount件の問題がありました。再入力して下さい。</p>" . &showregistform($editnum,$keyword,$redirecturl);
		&showadminpage("Reinput",'再入力',$msg,$BACKTOLINK. ' / <a href="?">初期画面に戻る</a>');
	}

}

# ---------------------- #　引数１： チェック文字列
# 登録申請の既存チェック #　引数２： 0=Keywordを対象, 1=URLを対象
# ---------------------- #　引数３： 除外データ番号(＝いま編集中の番号は重複チェックの対象外にする必要がある)	／戻り値： 0:マッチなし, 数値:マッチした登録No.
sub checkduplication
{
	my $trystring = shift @_;
	my $checkkind = shift @_;
	my $excludenum = shift @_;
	my $matchnum = 0;

	# 値の不正チェック
	if( $checkkind >= 2 || $checkkind < 0 ) { &errorexit("※checkduplicationに渡された引数checkkindの値がおかしい。"); }

	# 登録データ内を検索
	my $counter = 0;
	foreach my $oneline ( @rdata ) {
		$counter++;
		my @onedata = split(/,/,$oneline,2);
		$onedata[1] =~ s/(\r\n|\r|\n)$//g;	# 改行コードを削除しておく
		if( $trystring eq $onedata[$checkkind] ) {
			# 見つかったら
			if( $counter ne $excludenum ) {
				# 除外データ番号ではなければ、ループを抜ける
				$matchnum = $counter;
				last;
			}
		}
	}

	return $matchnum;
}

# ------------------ #
# データのリスト表示 #
# ------------------ #
sub showlist
{
	my @html;

	push(@html, qq|<p class="listguide">現在登録されている転送キーワードと転送先URLの対応は下記の通りです。</p><table class="list">| );
	push(@html, qq|<thead><tr><th></th><th>▼転送キーワード</th><th>▼転送先URL</th></tr></thead><tbody>\n| );
	my $counter = 0;	
	foreach my $oneline ( @rdata ) {
		$counter++;
		# レコード分割
		my ($key,$value) = split(/,/, $oneline, 2);
		$value =~ s/(\r\n|\r|\n)$//g;	# 改行コードを削除しておく
		# 表示用の安全策
		$key = &forsafety($key);
		$value = &forsafety($value);
		# テーブル出力
		push(@html, qq|<tr><th class="editno"><a href="?mode=edit&amp;edit=$counter" class="editno">$counter</a></th><td class="redirect"><a href="?$key" class="redirect" rel="nofollow">$key</a></td><td class="linkto"><a href="$value" class="linkto" rel="nofollow">$value</a></td></tr>\n| );
	}
	push(@html, qq|</tbody></table><p class="total">合計$counter個の転送キーワードが登録されています。(転送キーワードは大文字・小文字を区別します。)</p><ul class="guide"><li>登録情報を編集したい際は番号をクリック。</li><li>転送処理を試したい際は転送キーワードをクリック。</li><li>転送先を閲覧するにはURLをクリック。</li></ul>| );

	return @html;
}

# ------------- #
# URLのチェック #
# ------------- #
sub checkurl
{
	my $url = shift @_;

	if( $url eq "" ) {
		&showadminpage("ERROR",'Not Redirected','<p class="errorguide">登録されていないリダイレクト文字列が指定されたため、転送できませんでした。</p><p>※転送指定が誤っているか、またはこの転送設定は削除されました。詳しくはこのウェブサイトの管理者にお問い合わせ下さい。</p>',$BACKTOLINK);
		exit;
	}

	return $url;
}

# -------------- #
# 初期画面の表示 #
# -------------- #
sub startdisp
{
	my @listhtml = ();
	my $msg = '<p>ようこそ、Fumy Tiny URL Redirector へ！</p>';

	# --------------------
	# ▼登録フォームの表示
	# --------------------
	$msg .= &showregistform();

	# ----------------------------------
	# ▼登録リダイレクト先を一覧表示する
	# ----------------------------------
	if( $listshow == 1 ) {
		# 一覧表示する場合
		@listhtml = &showlist();
		$msg .= join("", @listhtml) . "\n" . '<p>※上記の登録内容一覧を<strong>表示したくない場合</strong>は、CGIソース内の設定値を変更して下さい。</p>';
	}
	else {
		# 一覧表示しない場合
		$msg .= '<p>※上記の登録フォームで「登録鍵」欄だけを入力して送信すれば、現在の登録内容一覧を表示します。</p>'
	}

	# ----------------
	# ▼管理画面を表示
	# ----------------
	&showadminpage("START",'',$msg,'<p class="notice">転送先の登録作業は、このCGIの管理者1人だけが行うことを前提に設計されています。不特定多数の人物が同時に登録作業を行える仕様にはなっていませんので、必ず1人の管理者だけが登録鍵を管理して登録作業を行って下さい。データファイルは適宜バックアップして下さい。CGIの各種動作設定は、CGIソースの冒頭に直接記載する仕様です。</p>');

	exit;
}

# -------------- #
# 編集画面の表示 #
# -------------- #
sub editpage
{
	my $targetnum = shift @_;
	my $targetkeyword = '';
	my $targeturl = '';

	my $msg;

	# 新規か編集か（編集なら現在の情報を取得する）
	if(( $targetnum > 0 ) && ( $targetnum <= ($#rdata + 1) )) {
		$msg = "<p>No.$targetnum の登録内容を編集します。</p>";
		($targetkeyword,$targeturl) = split(/,/,$rdata[$targetnum-1],2);
	}
	else {
		$msg = '<p>転送設定を新規に登録します。</p>';
		$targetnum = 0;
	}

	# --------------------
	# ▼登録フォームの表示
	# --------------------
	$msg .= &showregistform($targetnum,$targetkeyword,$targeturl);

	# ----------------
	# ▼管理画面を表示
	# ----------------
	&showadminpage("EDIT",'',$msg,'<a href="?">初期画面に戻る</a>');

	exit;
}

# ---------------- #
# 管理ページの表示 #
# -----------------#
sub showadminpage
{
	my ($title,$status,$body,$footer) = @_;
	my @adminhtml = <DATA>;
	foreach my $line (@adminhtml) {
		$line =~ s/_WORKTITLE_/$title/g;
		$line =~ s/_STATUS_/$status/g;
		$line =~ s/_BODY_/$body/g;
		$line =~ s/_FOOTER_/$footer/g;
		$line =~ s/_CHARCODE_/$charcode/g;
	}
	push(@adminhtml,&checkhtml(@adminhtml));

	print "Content-type: text/html\n\n";
	print @adminhtml;
}

# HTMLチェック
sub checkhtml {
	my $count = 0;
	foreach my $line (@_) {
		if( $line =~ m/<[aA]\s.*[hH][rR][eE][fF]=.*nishishi\.com.*>/ ) { $count++; }
	}
	if( $count > 0 ) { return "\n"; }
	else { return q|<div style="font-size:smaller;">- Powered by <a href="http://www.nishishi.com/">Fumy Tiny URL Redirector</a> -</div>|; }
}

# ------ #
# 安全用 #
# -------#
sub forsafety
{
	my $string 		= shift @_;

	# エンコード
	$string =~ s|&|&amp;|g;		# アンドを実体参照に
	$string =~ s|<|&lt;|g;		# 小なりを実体参照に
	$string =~ s|>|&gt;|g;		# 大なりを実体参照に
	$string =~ s|"|&quot;|g;	# 二重引用符を実体参照に
	$string =~ s|'|&apos;|g;	# 引用符を実体参照に

	return $string;
}

# ---------------- #
# エラーメッセージ #
# ---------------- #
sub errorexit
{
	my $msg = shift @_;

	print "Content-type: text/html\n\n";
	print qq|<!doctype html><html lang="ja"><head><title>Fumy Tiny URL Redirector ERROR</title><meta http-equiv="Content-Type" content="text/html; charset=$charcode"></head><body>\n|;
	print qq|<div style="background-color:red; color:#fffff0; font-weight:bold; font-family:Arial,sans-serif; padding:1px;"> Fumy Tiny URL Redirector ERROR </div><br>\n|;
	print qq|<div style="font-size: smaller;">エラー内容：</div>\n|;
	print qq|<div style="border:1px dotted blue; padding:1em;">$msg</div>\n|;
	print qq|<div style="font-size: 75%; font-family: Verdana,Arial,sans-serif; border-top: 1px solid #cccccc; padding-top: 0.3em;"><a href="http://www.nishishi.com/">Fumy Tiny URL Redirector</a></div>|;
	print qq|</body></html>\n|;

	exit;
}

__END__
<!DOCTYPE html>
<html lang="ja">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=_CHARCODE_">
	<meta name="viewport" content="initial-scale=1">
	<!-- この部分は、ＣＧＩのコントロールパネル用のＨＴＭＬです。 -->
	<title>_WORKTITLE_ - Fumy Tiny URL Redirector</title>
	<style type="text/css">
		body { background-color: #aaccaa; font-family: "メイリオ", Meiryo, "ヒラギノ角ゴ Pro W3", "Hiragino Kaku Gothic Pro", "ＭＳ Ｐゴシック", sans-serif; }
		div#contents { background-color: #ffffff; border: 1px #008000 solid; }
		div#header { background-color: #ccffcc; color: #000000; border-bottom: 1px green solid; }
		div#header h1 { margin: 0px 0.3em; font-size: 150%; font-family: "Lucida Sans Unicode","Microsoft Sans Serif","Century Gothic",sans-serif; }
		div#header h1 span.appname { font-size: 80%; }
		div#status { margin: 0px; padding: 0.3em 0.5em; text-align: right; font-size: 85%; }
		div#main { margin: 0.1em 1em 1em 1em; }
		div#footer { text-align: center; margin: 1em; }
		p#copyright { margin: 0.3em 0.5em; padding: 0.3em; font-size: 75%; }
		p.errorguide { background-color: #ffeeee; margin: 1em 0px; padding: 0.5em; font-weight: bold; color: #800; font-size: 1.1em; }
		p.successreport { background-color: #eeffee; margin: 1em 0px; padding: 0.5em; font-weight: bold; color: green; font-size: 1.1em; }
		.url { word-break: break-all; font-size: 0.9em; font-weight: normal; }

		table.list { border: 2px solid green; border-collapse: collapse; max-width: 100%; font-size: 0.9em; }
		table.list th, table.list td { border-color: green #bbeebb; border-width: 1px; border-style: solid; padding: 0px; }
		table.list thead th { background-color: #008000; color: white; border-color: green #eeffee; border-style: solid; padding: 0.2em 0.5em; }
		table.list th.editno { background-color: #80cc80; color: white; }
		table.list td.redirect { background-color: #eeffee; color: green; }
		table.list a { display: block; padding: 0.2em 0.5em; text-decoration: none; word-break: break-all; }
		table.list a.editno { color: white; }
		table.list a.redirect { color: darkblue; }
		table.list a:hover { text-decoration: underline; background-color: #00aa00; color: white; }

		.fieldform { margin: 1em 0px; border: 1px solid #ccc; border-radius: 12px; background-color: white; }
		.fieldform legend { background-color: #bbeebb; border: 1px solid #aaccaa; border-radius: 0.5em; padding: 0px 0.5em; font-weight: bold; }
		.fieldform p { margin: 0.5em 0px; }
		.fieldform label { display: inline-block; vertical-align: middle; min-width: 8.5em; }
		#keyword, #redirecturl, #registkey { border: 1px solid #aaa; border-radius: 0.25em; font-size: 1.2em; padding: 3px 0.25em; vertical-align: middle; width: 90%; }
		#showuselink { background-color: #eeffee; margin: 1em 0px; padding: 0.5em 1em; font-weight: bold; }
		.inputguide { font-size: 0.8em; vertical-align: middle; color: gray; display: inline-block; }
		#submitbutton { font-size: 1em; }
		.submenuitem { display: inline-block; font-size: 0.9em; }
		.notice { font-size: 0.8em; border: 1px solid pink; background-color: #ffdddd; text-align: left; padding: 0.75em 0.5em; }
		p.newrecord { margin-bottom: 1.5em; padding-bottom: 1em; border-bottom: 1px dashed gray; }
		p.demo { color: #cc0000; margin-bottom: 1.5em; padding-bottom: 1em; border-bottom: 1px dashed gray; } span.demo { background-color: #cc0000; color:white; font-weight: bold; padding: 1px 0.5em; }

		@media all and (min-width: 800px) {
			#keyword { width: 8em; }
			#redirecturl { width: 500px; }
			#registkey { width: 8em; }
			#submitbutton { font-size: 1.35em; }
			.submenuitem { margin-left: 1em; }
		}
		.hidedetail { display: none; }
		.important { color: #cc0000; }
	</style>
</head>
<body>
	<div id="contents">
		<div id="header">
			<h1>_WORKTITLE_<span class="appname"> - Fumy Tiny URL Redirector (Ver 2.0.0)</span></h1>
		</div>
		<div id="status">
			_STATUS_
		</div>
		<div id="main">
			_BODY_
		</div>
		<div id="footer">
			_FOOTER_
		</div>
	</div>
	<p id="copyright">
		<!-- ※以下の著作権表示を削除したり改変したりしないようにお願いします。 -->
		<a href="http://www.nishishi.com/cgi/tinyurl/">Fumy Tiny URL Redirector</a> Ver 2.0.0 : 
		Copyright &copy; 2006-2016 <a href="http://www.nishishi.com/">にしし/西村文宏(Nishishi)</a>.
	</p>
</body>
</html>
